"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _react = require("react");
var _AdvancedTextInputMaskListener = _interopRequireDefault(require("../../AdvancedTextInputMaskListener"));
var _CaretString = _interopRequireDefault(require("../../model/CaretString"));
var _types = require("../../model/types");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
const useMaskedTextInputListener = ({
  autocompleteOnFocus,
  mask,
  affinityFormat,
  affinityCalculationStrategy,
  customNotations,
  allowedKeys = "",
  autocomplete = true,
  autoSkip = false,
  isRTL = false,
  onChange,
  onChangeText,
  onTailPlaceholderChange,
  onFocus,
  defaultValue,
  validationRegex
}) => {
  const prevDispatchedPayload = (0, _react.useRef)({
    extracted: "",
    formatted: ""
  });
  const isInitialMount = (0, _react.useRef)(true);
  const [listener] = (0, _react.useState)(() => new _AdvancedTextInputMaskListener.default(mask, affinityFormat, customNotations, affinityCalculationStrategy, autocomplete, autoSkip, isRTL, allowedKeys, validationRegex, autocompleteOnFocus, defaultValue));
  (0, _react.useEffect)(() => {
    if (isInitialMount.current) {
      isInitialMount.current = false;
      return;
    }
    if (listener.affineFormats !== affinityFormat && affinityFormat) {
      // eslint-disable-next-line react-compiler/react-compiler
      listener.affineFormats = affinityFormat;
    }
    if (listener.customNotations !== customNotations && customNotations) {
      listener.customNotations = customNotations;
    }
    if (listener.affinityCalculationStrategy !== affinityCalculationStrategy && affinityCalculationStrategy !== undefined) {
      listener.affinityCalculationStrategy = affinityCalculationStrategy;
    }
    if (listener.autocomplete !== autocomplete) {
      listener.autocomplete = autocomplete;
    }
    if (listener.autoskip !== autoSkip) {
      listener.autoskip = autoSkip;
    }
    if (listener.primaryFormat !== mask) {
      listener.primaryFormat = mask;
    }
    if (listener.allowedKeys !== allowedKeys) {
      listener.setAllowedKeys(allowedKeys);
    }
    if (listener.rightToLeft !== isRTL) {
      listener.rightToLeft = isRTL;
    }
    if (listener.getValidationRegex() !== validationRegex) {
      listener.setValidationRegex(validationRegex);
    }
    if (listener.autocompleteOnFocus !== autocompleteOnFocus) {
      listener.autocompleteOnFocus = autocompleteOnFocus;
    }
    if (defaultValue !== listener.defaultValue) {
      listener.setText(defaultValue ?? "", false);
    }
  }, [autocompleteOnFocus, validationRegex, affinityFormat, customNotations, mask, isRTL, autoSkip, autocomplete, affinityCalculationStrategy, allowedKeys, listener, defaultValue]);
  const handleOnChange = (0, _react.useCallback)(e => {
    const {
      tailPlaceholder,
      formattedText,
      extractedValue,
      complete
    } = listener.handleTextChange(e);
    const formattedValue = formattedText.string;
    if (prevDispatchedPayload.current.extracted === extractedValue && prevDispatchedPayload.current.formatted === formattedValue) {
      return;
    }
    onChange === null || onChange === void 0 || onChange(e);
    onChangeText === null || onChangeText === void 0 || onChangeText(formattedValue, extractedValue, tailPlaceholder, complete);
    onTailPlaceholderChange === null || onTailPlaceholderChange === void 0 || onTailPlaceholderChange(tailPlaceholder);
    prevDispatchedPayload.current = {
      extracted: extractedValue,
      formatted: formattedValue
    };
  }, [listener, onChange, onChangeText, onTailPlaceholderChange]);
  const handleFocus = (0, _react.useCallback)(e => {
    listener.handleFocus(e);
    onFocus === null || onFocus === void 0 || onFocus(e);
  }, [listener, onFocus]);
  const defaultValueResult = (0, _react.useMemo)(() => defaultValue ? listener.primaryMask.apply(new _CaretString.default(defaultValue, defaultValue.length, {
    autocomplete: true,
    autoskip: false,
    type: _types.CaretGravityType.Forward
  })).formattedText.string : undefined, [defaultValue, listener.primaryMask]);
  return {
    setTextField: listener.setTextField,
    handleOnChange,
    handleFocus,
    listener,
    defaultValue: defaultValueResult,
    inputRef: listener.textField
  };
};
var _default = exports.default = useMaskedTextInputListener;
//# sourceMappingURL=index.js.map